#include <Keyboard.h>

#define KEY_PIN 3
#define ALT_IND 5
#define CTRL_IND 6
#define SHIFT_IND 7
#define WIN_IND 8
#define SPEAKER 9
#define DEBUG_PIN 15

uint8_t buf[8] = {
  0
};

int dotLength = 125;
unsigned long start_t, end_t;
String symbolBuffer = "";
bool Shift = false;
bool CTRL = false;
bool ALT = false;
bool WIN = false;
bool DEBUG;
int KEYCODE_A = 97;

void setup() {
  // put your setup code here, to run once:
  Keyboard.begin();
  pinMode(KEY_PIN, INPUT_PULLUP);
  pinMode(CTRL_IND, OUTPUT);
  pinMode(ALT_IND, OUTPUT);
  pinMode(WIN_IND, OUTPUT);
  pinMode(SHIFT_IND, OUTPUT);
  pinMode(DEBUG_PIN, INPUT);
  if (digitalRead(DEBUG_PIN) == HIGH){
    DEBUG = true;
    Serial.begin(9600);
    digitalWrite(CTRL_IND, HIGH);
    delay(250);
    digitalWrite(CTRL_IND, LOW);
    digitalWrite(ALT_IND, HIGH);
    delay(250);
    digitalWrite(ALT_IND, LOW);
    Serial.print("Morse");
    Serial.write("\t");
    Serial.print("Char");
    Serial.write("\n");
  }
  }
  

void loop() {
  while (!signalIncoming()) { }
  start_t = millis();
  tone(SPEAKER, 660);
  while (signalIncoming()) { }
  end_t = millis();
  noTone(SPEAKER); 

  unsigned long duration = end_t - start_t;

  delay(50);
  if (duration < 20) {
    return;
  }

  char morseChar = charFromSignalDuration(duration);
  symbolBuffer += morseChar;

  // Wait to see if we're continuing on the same character (symbol buffer)
  // If we fall out of this loop, we'll render the character
  while ((millis() - end_t) < dotLength * 3) {
    if (signalIncoming()) {
      return;
    }
  }

  // Convert and print char
  int index = morseToCharacterIndex(symbolBuffer);

  symbolBuffer = "";

  if (index < 0) {
    return;
  }
  sendChar(index);

  // Wait to see if we're continuing on the same word
  // If we fall out of this loop, we'll append a space
  while ((millis() - end_t) < dotLength * 7) {
    if (signalIncoming()) {
      return;
    }
  }

}

void sendChar(uint8_t code) {
  buf[2] = code;
  if (DEBUG){
    Serial.write(buf, 8);
    Serial.write("\n");
  }
  Keyboard.write(code);
  delay(100);
}

void debugPrint(String msg){
  if (DEBUG){
    Serial.print(msg);
    Serial.write("\n");
  }
}

int morseToCharacterIndex(const String& morse) {
  static char letters[][5] = {".-", "-...", "-.-.", "-..", ".",
                              "..-.", "--.", "....", "..", ".---",
                              "-.-", ".-..", "--", "-.", "---",
                              ".--.", "--.-", ".-.", "...", "-",
                              "..-", "...-", ".--", "-..-",
                              "-.--", "--..", "E"
                             };

  static char ascii[][7] = {".....-.", "......",".-..-.","...-.-","...---","..-...","..-.-.",".----.",
                               "-.--.", "-.--.-", ".-...-", ".-.-.", "--..--",
                               ".----.", ".-.-.-", "-..-.", "-----",".----", "..---", "...--", "....-",
                              ".....", "-....", "--...", "---..",
                              "----.", "---...", "-.-.-.", "..--.-", "-...-", "..----",
                               "..--..", ".--.-.", "E"
                           };

  static char symbols3[][7] = {"....--", ".-....", "...-..", "..-..-", "..-.--",
                               ".-..-.", "E"
                              };
                              
  static char symbols4[][7] = {".....-", ".-...-", "....-.", ".-....", "E"
                              };

  static char mods[][8] = {".......","......-",".....-.",".....--", "....-..", "....-.-", "----", "E"};
  

  if (DEBUG) {
    Serial.print(morse);
    Serial.print("\t");
  }

  int i = 0;
  while (String(letters[i]) != "E") {
    if (String(letters[i]) == morse) {
      return KEYCODE_A + i;
    }
    i++;
  }

  int j = 0;
  while (String(ascii[j]) != "E") {
    if (String(ascii[j]) == morse) {
      return 32 + j;
    }
    j++;
  }

  j = 0;
  while (String(symbols3[j]) != "E") {
    if (String(symbols3[j]) == morse) {
      return 91 + j;
    }
    j++;
  }

  j = 0;
  while (String(symbols4[j]) != "E") {
    if (String(symbols4[j]) == morse) {
      return 123 + j;
    }
    j++;
  }
  
  j = 0;
  while (String(mods[j]) != "E") {
    if (String(mods[j]) == morse) {
      switch(j){
        case 6:
          debugPrint("BKSP");
          Keyboard.press(KEY_BACKSPACE);
          Keyboard.release(KEY_BACKSPACE);
          break;
        case 5:
          if (!WIN){
            debugPrint("WIN");
            WIN = true;
            Keyboard.press(KEY_LEFT_GUI);
            digitalWrite(WIN_IND, HIGH);
          }
          else{
            debugPrint("Un-WIN");
            WIN = false;
            Keyboard.release(KEY_LEFT_GUI);
            digitalWrite(WIN_IND, LOW);
          }
          break;   
        case 4:
          if (!ALT){
            debugPrint("ALT");
            ALT = true;
            Keyboard.press(KEY_LEFT_ALT);
            digitalWrite(ALT_IND, HIGH);
          }
          else{
            debugPrint("Un-ALT");
            ALT = false;
            Keyboard.release(KEY_LEFT_ALT);
            digitalWrite(ALT_IND, LOW);
          }
          break;        
        case 3:
          if (!CTRL){
            debugPrint("CTRL");
            CTRL = true;
            Keyboard.press(KEY_LEFT_CTRL);
            digitalWrite(CTRL_IND, HIGH);
          }
          else{
            debugPrint("Un-CTRL");
            CTRL = false;
            Keyboard.release(KEY_LEFT_CTRL);
            digitalWrite(CTRL_IND, LOW);
          }
          break;
        case 2:
          debugPrint("Space");
          return 32;
          break;
        case 1:
          if (!Shift){
            debugPrint("Shift");
            Keyboard.press(KEY_LEFT_SHIFT);
            Shift = true;
            KEYCODE_A = 65;
            digitalWrite(SHIFT_IND, HIGH);
          }
          else{
            debugPrint("Un-Shift");
            Keyboard.release(KEY_LEFT_SHIFT);
            Shift = false;
            KEYCODE_A = 97;
            digitalWrite(SHIFT_IND, LOW);
          }
          break;
        case 0:
          debugPrint("Enter");
          Keyboard.press(KEY_RETURN);
          Keyboard.release(KEY_RETURN);
          break;
      }
    }
    j++;
  }


  return -1;
}


char charFromSignalDuration(unsigned long duration) {
  return (duration < dotLength * 1.5) ? '.' : '-';
}

bool signalIncoming() {
  return digitalRead(KEY_PIN) == LOW;
}
